import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="neatly",
    version="0.0.20",
    author="Sean Rhee",
    author_email="seanrhee2024@u.northwestern.edu",
    description="A NEAT package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/s2011r2593/neatly",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
)